import random
import string
import traceback
from ._types import Tweet, UserTweets, Media,ShortUser,User,Search,Trends,UserLegacy
from bs4 import BeautifulSoup as bs
import requests as s
UserTweets = UserTweets
User = User
Search = Search
Trends = Trends
UserLegacy = UserLegacy


def proxyFactory() -> list:
    # https://proxylist.geonode.com/api/organdasn?limit=200&page=2
    r2 = s.get("https://free-proxy-list.net/")
    soup = bs(r2.content, "html.parser")
    tds = soup.find("tbody").find_all("tr")
    proxies = []
    for io in tds:
        proxies.append(f"{io.find_all('td')[0].text}:{io.find_all('td')[1].text}")
    return proxies


def get_graph_ql_query(typed, user, pages=None) -> str:
    """

    :param typed: internal script query type
    :param user: username or user_id
    :param pages: cursor of next page
    :return: string on graphql query object
    """
    if typed == 1:
        """
        {
            "userId":"",
            "count":20,
            "cursor":""
            "withTweetQuoteCount":true,
            "includePromotedContent":true,
            "withSuperFollowsUserFields":false,
            "withUserResults":true,
            "withBirdwatchPivots":false,
            "withReactionsMetadata":false,
            "withReactionsPerspective":false,
            "withSuperFollowsTweetFields":false,
            "withVoice":true
        }
        """
        if pages:
            data = '''%7B%22userId%22%3A%22''' + user + '''%22%2C%22count%22%3A20%2C%22cursor%22%3A%22''' + pages + '''%22%2C%22withTweetQuoteCount%22%3Atrue%2C%22includePromotedContent%22%3Atrue%2C%22withSuperFollowsUserFields%22%3Afalse%2C%22withUserResults%22%3Atrue%2C%22withBirdwatchPivots%22%3Afalse%2C%22withReactionsMetadata%22%3Afalse%2C%22withReactionsPerspective%22%3Afalse%2C%22withSuperFollowsTweetFields%22%3Afalse%2C%22withVoice%22%3Atrue%7D'''
        else:
            data = '''%7B%22userId%22%3A%22''' + user + '''%22%2C%22count%22%3A20%2C%22withTweetQuoteCount%22%3Atrue%2C%22includePromotedContent%22%3Atrue%2C%22withSuperFollowsUserFields%22%3Afalse%2C%22withUserResults%22%3Atrue%2C%22withBirdwatchPivots%22%3Afalse%2C%22withReactionsMetadata%22%3Afalse%2C%22withReactionsPerspective%22%3Afalse%2C%22withSuperFollowsTweetFields%22%3Afalse%2C%22withVoice%22%3Atrue%7D'''
    elif typed == 2:
        if pages:
            data = '''%7B%22userId%22%3A%22''' + user + '''%22%2C%22count%22%3A40%2C%22cursor%22%3A%22''' + pages + '''%22%2C%22includePromotedContent%22%3Atrue%2C%22withCommunity%22%3Atrue%2C%22withSuperFollowsUserFields%22%3Atrue%2C%22withDownvotePerspective%22%3Afalse%2C%22withReactionsMetadata%22%3Afalse%2C%22withReactionsPerspective%22%3Afalse%2C%22withSuperFollowsTweetFields%22%3Atrue%2C%22withVoice%22%3Atrue%2C%22withV2Timeline%22%3Afalse%2C%22__fs_interactive_text%22%3Afalse%2C%22__fs_responsive_web_uc_gql_enabled%22%3Afalse%2C%22__fs_dont_mention_me_view_api_enabled%22%3Afalse%7D'''
        else:
            data = '''%7B%22userId%22%3A%2244196397%22%2C%22count%22%3A40%2C%22includePromotedContent%22%3Atrue%2C%22withCommunity%22%3Atrue%2C%22withSuperFollowsUserFields%22%3Atrue%2C%22withDownvotePerspective%22%3Afalse%2C%22withReactionsMetadata%22%3Afalse%2C%22withReactionsPerspective%22%3Afalse%2C%22withSuperFollowsTweetFields%22%3Atrue%2C%22withVoice%22%3Atrue%2C%22withV2Timeline%22%3Afalse%2C%22__fs_interactive_text%22%3Afalse%2C%22__fs_responsive_web_uc_gql_enabled%22%3Afalse%2C%22__fs_dont_mention_me_view_api_enabled%22%3Afalse%7D'''
    else:
        """
        {
             "screen_name":f"{user}",
             "withSafetyModeUserFields":True,
             "withSuperFollowsUserFields":False
         }
        """
        data = '''%7B%22screen_name%22%3A%22''' + user + '''%22%2C%22withSafetyModeUserFields%22%3Atrue%2C%22withSuperFollowsUserFields%22%3Afalse%7D'''
    return data


def get_headers(typed=None) -> dict:
    if not typed:
        headers = {
            "authority": "twitter.com",
            "sec-ch-ua": '"Google Chrome";v="93", " Not;A Brand";v="99", "Chromium";v="93"',
            "x-twitter-client-language": "en",
            "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "accept-encoding": "gzip, deflate, br",
            "upgrade-insecure-requests": "1",
            "sec-ch-ua-platform": 'Windows"',
            "sec-ch-ua-mobile": "?0",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.82 Safari/537.36",
        }
    else:
        headers = {
            'x-csrf-token': ''.join(random.choices(string.ascii_letters + string.digits, k=32)),
            'authorization': "Bearer " + "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA",
            'content-type': "application/json",
            'referer': "https://twitter.com/AmitabhJha3",
            "authority": "twitter.com",
            "sec-ch-ua": '"Google Chrome";v="93", " Not;A Brand";v="99", "Chromium";v="93"',
            "x-twitter-client-language": "en",
            "upgrade-insecure-requests": "1",
            "sec-ch-ua-platform": 'Windows"',
            "sec-ch-ua-mobile": "?0",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.82 Safari/537.36",
            "x-guest-token": typed
        }
    return headers


def format_search(response,simplify):
    tweet = {
        "result": {
            "tweets": []
        }
    }
    __cursor = []
    for i in response.json()['timeline']['instructions'][0]['addEntries']['entries']:
        try:
            if i['content']['operation']:
                if i['content']['operation']['cursor']['cursorType'] == "Bottom":
                    __cursor.append(i['content']['operation']['cursor']['value'])
        except:
            pass
    users = response.json()['globalObjects']['users']
    for i in response.json()['globalObjects']['tweets']:
        if simplify:
            tweet_ = Tweet(
                simplify_tweet(
                    response.json()['globalObjects']['tweets'][i],
                    response.json()['globalObjects']['tweets'][i]['id'],
                    users.get(str(response.json()['globalObjects']['tweets'][i]['user_id'])),
                    True
                )
            )
            tweet['result']['tweets'].append(tweet_)
        else:
            tweet['result']['tweets'].append(response.json()['globalObjects']['tweets'][i])
    try:
        if not __cursor:
            for i in response.json()['timeline']['instructions']:
                for key in i.keys():
                    if key == "replaceEntry":
                        if i['replaceEntry']['entry']['content']['operation']['cursor']['cursorType'] == "Bottom":
                            __cursor.append(i['replaceEntry']['entry']['content']['operation']['cursor']['value'])
                            break
    except:
        pass
    return tweet, __cursor


def simplify_tweet(tweet, rest_id,author,author_legacy=False):
    try:
        created_on = tweet['created_at'] if tweet['created_at'] else ""
    except KeyError:
        created_on = ""
    rest_id = rest_id
    try:
        is_retweet = True if str(tweet['full_text']).startswith("RT") else False
    except:
        is_retweet = False
    try:
        if is_retweet:
            text = tweet['retweeted_status_result']['result']['legacy']['full_text']
        else:
            text = tweet['full_text'] if tweet['full_text'] else ""
    except KeyError:
        text = ""
    try:
        is_reply = True if tweet.get('in_reply_to_status_id') is not None or tweet.get('in_reply_to_user_id') is not None or tweet.get('in_reply_to_screen_name') is not None else False
    except:
        is_reply = False
    if is_reply:
        reply_to = tweet['in_reply_to_screen_name']
    try:
        language = tweet['lang'] if tweet['lang'] else ""
    except KeyError:
        language = ""
    try:
        likes = tweet['favorite_count'] if tweet['favorite_count'] else ""
    except KeyError:
        likes = ""
    try:
        retweet_count = tweet['retweet_count'] if tweet['retweet_count'] else ""
    except KeyError:
        retweet_count = ""
    try:
        source = str(tweet['source']).split(">")[1].split("<")[0] if tweet['source'] else ""
    except KeyError:
        source = ""
    try:
        media = [Media(i) for i in tweet['entities']['media']] if tweet.get('entities').get('media') else None
    except KeyError:
        media = ""
    try:
        mentions = [ShortUser(l) for l in tweet['entities']['user_mentions']] if tweet.get('entities').get('user_mentions') else None
    except KeyError:
        mentions = ""
    try:
        urls = tweet['entities']['urls'] if tweet['entities']['urls'] else ""
    except KeyError:
        urls = ""
    try:
        hashtags = tweet['entities']['hashtags'] if tweet['entities']['hashtags'] else ""
    except KeyError:
        hashtags = ""
    try:
        symbols = tweet['entities']['symbols'] if tweet['entities']['symbols'] else ""
    except KeyError:
        symbols = ""
    if not author_legacy:
        author_ = User(author,2)
    else:
        author_ = UserLegacy(author)
    result = {
        "created_on": created_on,
        "author":author_,
        "is_retweet": is_retweet,
        "is_reply":is_reply,
        "tweet_id": rest_id,
        "tweet_body": text,
        "language": language,
        "likes": likes,
        "retweet_counts": retweet_count,
        "source": source,
        "media": media,
        "user_mentions": mentions,
        "urls": urls,
        "hashtags": hashtags,
        "symbols": symbols
    }
    if is_reply:
        result['reply_to'] = reply_to
    return result


def format_tweet_json(response, include_extras, simplify):
    tweet = {
        "result": {
            "tweets": []
        }
    }
    __cursor = []
    for i in response.json()['data']['user']['result']['timeline']['timeline']['instructions'][0]['entries']:
        if str(i['entryId']).split("-")[0] == "tweet":
            try:
                tweet['result']['tweets'].append(
                    Tweet(
                        simplify_tweet(
                            i['content']['itemContent']['tweet_results']['result']['legacy'],
                            i['content']['itemContent']['tweet_results']['result']['rest_id'],
                            i['content']['itemContent']['tweet_results']['result']['core']
                        )
                    )
                )
            except:
                pass
        elif str(i['entryId']).split("-")[0] == "cursor":
            if i['content']['cursorType'] == "Bottom":
                __cursor.append(i['content']['value'])
        else:
            if include_extras is True:
                if str(i['entryId']).split("-")[0] in tweet['result']:
                    pass
                else:
                    tweet['result'][f"{str(i['entryId']).split('-')[0]}"] = []
                tweet['result'][f"{str(i['entryId']).split('-')[0]}"].append(i)
            else:
                pass
    return tweet, __cursor






import re
from .utils import *
from .exceptions_ import *
import traceback
import requests as s
import sys


class Twitter:
    def __init__(self, profile_name=None):
        if profile_name:
            if profile_name.startswith("https://"):
                self.profile_url = profile_name
            else:
                self.profile_url = f"https://twitter.com/{profile_name}"
        else:
            self.profile_url = None
        self.__user_by_screen_url = "https://twitter.com/i/api/graphql/B-dCk4ph5BZ0UReWK590tw/UserByScreenName?variables="
        self.__tweets_url = "https://twitter.com/i/api/graphql/Lya9A5YxHQxhCQJ5IPtm7A/UserTweets?variables="
        self.__tweets_with_replies = "https://twitter.com/i/api/graphql/B9izm_qt4l5qWUWrympCVw/UserTweetsAndReplies?variables="
        self.__trends_url = "https://api.twitter.com/2/guide.json?include_profile_interstitial_type=1&include_blocking=1&include_blocked_by=1&include_followed_by=1&include_want_retweets=1&include_mute_edge=1&include_can_dm=1&include_can_media_tag=1&skip_status=1&cards_platform=Web-12&include_cards=1&include_ext_alt_text=true&include_quote_count=true&include_reply_count=1&tweet_mode=extended&include_entities=true&include_user_entities=true&include_ext_media_color=true&include_ext_media_availability=true&send_error_codes=true&simple_quoted_tweet=true&count=20&candidate_source=trends&include_page_configuration=false&entity_tokens=false&ext=mediaStats%2ChighlightedLabel"
        self.__search_url = "https://twitter.com/i/api/2/search/adaptive.json?include_profile_interstitial_type=1&include_blocking=1&include_blocked_by=1&include_followed_by=1&include_want_retweets=1&include_mute_edge=1&include_can_dm=1&include_can_media_tag=1&skip_status=1&cards_platform=Web-12&include_cards=1&include_ext_alt_text=true&include_quote_count=true&include_reply_count=1&tweet_mode=extended&include_entities=true&include_user_entities=true&include_ext_media_color=true&include_ext_media_availability=true&send_error_codes=true&simple_quoted_tweet=true&q={}&count=20&query_source=typeahead_click&pc=1&spelling_corrections=1&ext=mediaStats%2ChighlightedLabel%2CvoiceInfo"
        self.__tweet_detail_url = "https://twitter.com/i/api/graphql/4tzuTRu5-fpJTS7bDF6Nlg/TweetDetail?variables=%7B%22focalTweetId%22%3A%22{}%22%2C%22with_rux_injections%22%3Afalse%2C%22includePromotedContent%22%3Atrue%2C%22withCommunity%22%3Atrue%2C%22withTweetQuoteCount%22%3Atrue%2C%22withBirdwatchNotes%22%3Afalse%2C%22withSuperFollowsUserFields%22%3Afalse%2C%22withUserResults%22%3Atrue%2C%22withBirdwatchPivots%22%3Afalse%2C%22withReactionsMetadata%22%3Afalse%2C%22withReactionsPerspective%22%3Afalse%2C%22withSuperFollowsTweetFields%22%3Afalse%2C%22withVoice%22%3Atrue%7D"
        self.__guest_token_url = "https://api.twitter.com/1.1/guest/activate.json"
        self.__proxy = {"http": random.choice(proxyFactory())}
        self.__guest_token = self.__get_guest_token()
        self.__guest_headers = get_headers(self.__guest_token)

    def __get_guest_token(self, max_retries=10):
        try:
            guest_token = ""
            for i in range(0, int(max_retries)):
                if self.profile_url:
                    response = s.get(self.profile_url, headers=get_headers(), proxies=self.__proxy)
                else:
                    response = s.get("https://twitter.com/i/trends", headers=get_headers(), proxies=self.__proxy)
                guest_token_ = re.findall('document\.cookie = decodeURIComponent\("gt=(.*?); Max-Age=10800; Domain=\.twitter\.com; Path=/; Secure"\);',response.text)
                try:
                    if guest_token_[0]:
                        guest_token = guest_token_[0]
                        return guest_token
                except IndexError:
                    try:
                        headers = get_headers()
                        headers['x-csrf-token'] = ''.join(random.choices(string.ascii_letters + string.digits, k=32))
                        headers['authorization'] = "Bearer " + "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs%3D1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
                        headers['content-type'] = 'application/x-www-form-urlencoded'
                        headers['accept'] = "*/*"
                        response = s.post(self.__guest_token_url,headers=headers,proxies=self.__proxy)
                        guest_token = response.json()['guest_token']
                        return response.json()['guest_token']
                    except:
                        continue
            if guest_token == "":
                sys.exit(f"Script Aborting : Guest Token couldn't be found after {max_retries} retires.")
        except Exception as e:
            traceback.print_exc()
            sys.exit(f"Script Aborting : Guest Token couldn't be found after {max_retries} retires.\n{e}")

    def __verify_user(self):
        user = self.profile_url.split("/")[-1]
        data = str(get_graph_ql_query(3, user))
        response = s.get(
            f"{self.__user_by_screen_url}{data}",
            headers=self.__guest_headers,
            proxies=self.__proxy
        )
        if response.json().get('data'):
            return response.json()
        else:
            return 0

    def get_user_info(self, banner_extensions=False, image_extensions=False):
        if self.profile_url:
            json_ = self.__verify_user()
            if json_ == 0:
                raise UserNotFound()
            else:
                if not banner_extensions or banner_extensions is False:
                    del json_['data']['user']['result']['legacy']['profile_banner_extensions']
                if not image_extensions or image_extensions is False:
                    del json_['data']['user']['result']['legacy']['profile_image_extensions']
                return User(json_)
        else:
            raise ValueError("No Username Provided , Please initiate the class using a username or profile URL")

    def get_user_id(self):
        if self.profile_url:
            user = self.__verify_user()
            if user == 0:
                raise UserNotFound()
            else:
                return User(user).rest_id
        else:
            raise ValueError("No Username Provided , Please initiate the class using a username or profile URL")

    def get_tweets(self,pages=1,include_extras=False,replies=False):
        try:
            if self.profile_url:
                user_id = self.get_user_id()
                result = {"tweets":[]}
                __nextCursor = None
                for page in range(0,int(pages)):
                    if replies:
                        data = str(get_graph_ql_query(2, user_id,__nextCursor))
                        response = s.get(f"{self.__tweets_with_replies}{data}", headers=self.__guest_headers,
                                         proxies=self.__proxy)
                    else:
                        data = str(get_graph_ql_query(1, user_id,__nextCursor))
                        response = s.get(f"{self.__tweets_url}{data}", headers=self.__guest_headers,
                                         proxies=self.__proxy)
                    tweet,__Cursor = format_tweet_json(response,include_extras=include_extras,simplify=True)
                    for i in tweet['result']['tweets']:
                        result['tweets'].append(i)
                    if __nextCursor != __Cursor:
                        __nextCursor = __Cursor[0]
                    else:
                        break
                return UserTweets(result,user_id)
            else:
                raise ValueError("No Username Provided , Please initiate the class using a username or profile URL")
        except:
            traceback.print_exc()
            exit(1)

    def get_trends(self) -> dict:
        trends = {"trends":[]}
        response = s.get(f"{self.__trends_url}", headers=self.__guest_headers,
                         proxies=self.__proxy)
        for i in response.json()['timeline']['instructions'][1]['addEntries']['entries'][1]['content']['timelineModule']['items']:
            data = {
                "name":i['item']['content']['trend']['name'],
                "url": str(i['item']['content']['trend']['url']['url']).replace("twitter://","https://twitter.com/").replace("query","q"),
            }
            try:
                if i['item']['content']['trend']['trendMetadata']['metaDescription']:
                    data['tweet_count'] = i['item']['content']['trend']['trendMetadata']['metaDescription']
            except:
                pass
            trends['trends'].append(Trends(data))
        return trends

    def search(self, keyword, pages=1, latest=False):
        result = {"tweets":[]}
        if keyword.startswith("#"):
            keyword = f"%23{keyword[1:]}"
        nextCursor = None
        for page in range(0,int(pages)):
            if nextCursor:
                search_url = f"{self.__search_url}&cursor={nextCursor}"
            else:
                search_url = self.__search_url
            if latest is False:
                r = s.get(search_url.format(keyword), headers=self.__guest_headers, proxies=self.__proxy)
            else:
                url = f"{search_url}&tweet_search_mode=live"
                r = s.get(url.format(keyword), headers=self.__guest_headers, proxies=self.__proxy)
            tweets_, __cursor = format_search(r, True)
            for i in tweets_['result']['tweets']:
                result['tweets'].append(i)
            if __cursor != nextCursor:
                nextCursor = __cursor[0]
            else:
                break
        return Search(result,keyword)

    def tweet_detail(self,identifier):
        if str(identifier).startswith("https://"):
            if str(identifier).endswith("/"):
                tweetId = str(identifier)[:-1].split("/")[-1]
            else:
                tweetId = str(identifier).split("/")[-1]
        else:
            tweetId = identifier
        result = {
            "conversation_threads":[]
        }
        r = s.get(self.__tweet_detail_url.format(tweetId), headers=self.__guest_headers, proxies=self.__proxy)
        for entry in r.json()['data']['threaded_conversation_with_injections']['instructions'][0]['entries']:
            if str(entry['entryId']).split("-")[0] == "tweet":
                tweet = entry['content']['itemContent']['tweet_results']['result']['legacy']
                result['tweet'] = tweet
            else:
                result['conversation_threads'].append(entry)
        return result



import csv
import openpyxl

WORKBOOK_HEADERS = ['Created on','author', 'is_retweet', 'is_reply', 'tweet_id', 'tweet_body', 'language', 'likes',
                    'retweet_count', 'source', 'medias', 'user_mentioned', 'urls', 'hashtags', 'symbols']


class UserTweets:
    def __init__(self, dictionary,user):
        self.dict_ = dictionary
        self.user = user

    def to_xlsx(self, filename=None):
        if self.dict_.get("error") is None:
            wb = openpyxl.Workbook()
            ws = wb.active
            for v, i in enumerate(WORKBOOK_HEADERS):
                ws.cell(row=1, column=v + 1).value = i
            max_row = 1
            all_tweets = self.dict_['tweets']
            for p in all_tweets:
                p = p.to_dict()
                ws[f'A{max_row + 1}'] = p['created_on']
                ws[f'B{max_row + 1}'] = p['author'].name
                ws[f'C{max_row + 1}'] = p['is_retweet']
                ws[f'D{max_row + 1}'] = p['is_reply']
                ws[f'E{max_row + 1}'] = p['tweet_id']
                ws[f'F{max_row + 1}'] = p['tweet_body']
                ws[f'G{max_row + 1}'] = p['language']
                ws[f'H{max_row + 1}'] = p['likes']
                ws[f'I{max_row + 1}'] = p['retweet_counts']
                ws[f'J{max_row + 1}'] = p['source']
                media_ = ""
                if not isinstance(p['media'], str) and p['media'] is not None:
                    for media in p['media']:
                        media_ = f"{media_} {media.expanded_url};"
                    ws[f'K{max_row + 1}'] = media_
                else:
                    ws[f'K{max_row + 1}'] = p['media']
                user_mentions = ""
                if not isinstance(p['user_mentions'], str) and p['user_mentions'] is not None:
                    for user in p['user_mentions']:
                        user_mentions = f"{user_mentions} {user.screen_name};"
                    ws[f'L{max_row + 1}'] = user_mentions
                else:
                    ws[f'L{max_row + 1}'] = p['user_mentions']
                urls = ""
                if not isinstance(p['urls'], str):
                    for url in p['urls']:
                        urls = f"{urls} {url['expanded_url']};"
                    ws[f'M{max_row + 1}'] = urls
                else:
                    ws[f'M{max_row + 1}'] = p['urls']
                hashtags = ""
                if not isinstance(p['hashtags'], str):
                    for tag in p['hashtags']:
                        hashtags = f"{hashtags} {tag['text']};"
                    ws[f'N{max_row + 1}'] = hashtags
                else:
                    ws[f'N{max_row + 1}'] = p['hashtags']
                ws[f'O{max_row + 1}'] = p['symbols']
                max_row = max_row + 1
            if not filename:
                filename = f"tweets-{self.user}.xlsx"
            wb.save(filename)
        else:
            return self.dict_['error']

    def to_csv(self,filename=None):
        all_tweets = self.dict_['tweets']
        if not filename:
            filename = f"tweets-{self.user}.csv"
        with open(filename,"w",encoding="ASCII",errors="ignore",newline="") as csv_:
            writer = csv.writer(csv_)
            writer.writerow(WORKBOOK_HEADERS)
            for p in all_tweets:
                p = p.to_dict()
                created_on = p.get('created_at') if p.get('created_at') else p.get('created_on')
                author = p['author'].name
                is_retweet = p['is_retweet']
                is_reply = p['is_reply']
                tweet_id = p['tweet_id']
                tweet_body = p['tweet_body']
                language = p['language']
                likes = p['likes']
                retweet_count = p['retweet_counts']
                source = p['source']
                symbols = p['symbols']
                media_ = ""
                if not isinstance(p['media'], str) and p['media'] is not None:
                    for media in p['media']:
                        media_ = f"{media_} {media.expanded_url};"
                else:
                    media_ = p['media']
                user_mentions = ""
                if not isinstance(p['user_mentions'], str) and p['user_mentions'] is not None:
                    for user in p['user_mentions']:
                        user_mentions = f"{user_mentions} {user.screen_name};"
                else:
                    user_mentions = p['user_mentions']
                hashtags = ""
                if not isinstance(p['hashtags'], str):
                    for tag in p['hashtags']:
                        hashtags = f"{hashtags} {tag['text']};"
                else:
                    hashtags = p['hashtags']
                urls = ""
                if not isinstance(p['urls'], str):
                    for url in p['urls']:
                        urls = f"{urls} {url['expanded_url']};"
                else:
                    urls = p['urls']
                row = [created_on,author,is_retweet,is_reply,tweet_id,tweet_body,language,likes,retweet_count,source,media_,user_mentions,urls,hashtags,symbols]
                writer.writerow(row)

    def to_dict(self):
        return self.dict_

    def __repr__(self):
        return f"<UserTweets (user={self.user}) (count={len(self.dict_['tweets'])})>"


class Tweet:
    def __init__(self,tweet_dict):
        self.__dictionary = tweet_dict
        self.id = self.__dictionary.get("tweet_id")
        self.author = self.__dictionary.get("author")
        self.created_on = self.__dictionary.get("created_on")
        self.is_retweet = self.__dictionary.get("is_retweet")
        self.is_reply = self.__dictionary.get("is_reply")
        self.tweet_body = self.__dictionary.get("tweet_body")
        self.language = self.__dictionary.get("language")
        self.likes = self.__dictionary.get("likes")
        self.retweet_counts = self.__dictionary.get("retweet_counts")
        self.media = self.__dictionary.get("media")
        self.user_mentions = self.__dictionary.get("user_mentions")
        self.urls = self.__dictionary.get("urls")
        self.hashtags = self.__dictionary.get("hashtags")
        self.symbols = self.__dictionary.get("symbols")
        self.reply_to = self.__dictionary.get("reply_to")

    def __repr__(self):
        return f"<Tweet (id={self.id}) (author={self.author}) (created_on={self.created_on})>"

    def to_dict(self):
        return self.__dictionary


class Media:
    def __init__(self,media_dict):
        self.__dictionary = media_dict
        self.display_url = self.__dictionary.get("display_url")
        self.expanded_url = self.__dictionary.get("expanded_url")
        self.id = self.__dictionary.get("id_str")
        self.indices = self.__dictionary.get("indices")
        self.media_url_https = self.__dictionary.get("media_url_https")
        self.type = self.__dictionary.get("type")
        self.url = self.__dictionary.get("url")
        self.features = self.__dictionary.get("features")
        self.sizes = self.__dictionary.get("sizes")
        self.original_info = self.__dictionary.get("original_info")

    def __repr__(self):
        return f"<Media (id={self.id}) (type={self.type})>"

    def to_dict(self):
        return self.__dictionary


class ShortUser:
    def __init__(self,user_dict):
        self.__dictionary = user_dict
        self.id = self.__dictionary.get("id_str")
        self.name = self.__dictionary.get("name")
        self.screen_name = self.__dictionary.get("screen_name")

    def __repr__(self):
        return f"<ShortUser (id={self.id}) (name={self.name})>"

    def to_dict(self):
        return self.__dictionary


class User:
    def __init__(self,user_dict,type_=1):
        self.__dictionary = user_dict['data']['user']['result'] if type_ == 1 else user_dict['user_results']['result']
        self.id = self.__dictionary.get("id")
        self.rest_id = self.__dictionary.get("rest_id")
        self.created_at = self.__dictionary.get("legacy").get("created_at")
        self.default_profile = self.__dictionary.get("legacy").get("default_profile")
        self.default_profile_image = self.__dictionary.get("legacy").get("default_profile_image")
        self.description = self.__dictionary.get("legacy").get("description")
        self.entities = self.__dictionary.get("legacy").get("entities")
        self.fast_followers_count = self.__dictionary.get("legacy").get("fast_followers_count")
        self.favourites_count = self.__dictionary.get("legacy").get("favourites_count")
        self.followers_count = self.__dictionary.get("legacy").get("followers_count")
        self.friends_count = self.__dictionary.get("legacy").get("friends_count")
        self.has_custom_timelines = self.__dictionary.get("legacy").get("has_custom_timelines")
        self.is_translator = self.__dictionary.get("legacy").get("is_translator")
        self.listed_count = self.__dictionary.get("legacy").get("listed_count")
        self.location = self.__dictionary.get("legacy").get("location")
        self.media_count = self.__dictionary.get("legacy").get("media_count")
        self.name = self.__dictionary.get("legacy").get("name")
        self.normal_followers_count = self.__dictionary.get("legacy").get("normal_followers_count")
        self.profile_banner_url = self.__dictionary.get("legacy").get("profile_banner_url")
        self.profile_image_url_https = self.__dictionary.get("legacy").get("profile_image_url_https")
        self.profile_interstitial_type = self.__dictionary.get("legacy").get("profile_interstitial_type")
        self.protected = self.__dictionary.get("legacy").get("protected")
        self.screen_name = self.__dictionary.get("legacy").get("screen_name")
        self.statuses_count = self.__dictionary.get("legacy").get("statuses_count")
        self.translator_type = self.__dictionary.get("legacy").get("translator_type")
        self.verified = self.__dictionary.get("legacy").get("verified")

    def __repr__(self):
        return f"<User (id={self.rest_id}) (name={self.name}) (followers={self.followers_count}) (verified={self.verified})>"

    def to_dict(self):
        return self.__dictionary


class Search:
    def __init__(self, dictionary, keyword):
        self.dict_ = dictionary
        self.keyword = keyword

    def to_xlsx(self, filename=None):
        if self.dict_.get("error") is None:
            wb = openpyxl.Workbook()
            ws = wb.active
            for v, i in enumerate(WORKBOOK_HEADERS):
                ws.cell(row=1, column=v + 1).value = i
            max_row = 1
            all_tweets = self.dict_['tweets']
            for p in all_tweets:
                p = p.to_dict()
                ws[f'A{max_row + 1}'] = p['created_on']
                ws[f'B{max_row + 1}'] = p['author'].name
                ws[f'C{max_row + 1}'] = p['is_retweet']
                ws[f'D{max_row + 1}'] = p['is_reply']
                ws[f'E{max_row + 1}'] = p['tweet_id']
                ws[f'F{max_row + 1}'] = p['tweet_body']
                ws[f'G{max_row + 1}'] = p['language']
                ws[f'H{max_row + 1}'] = p['likes']
                ws[f'I{max_row + 1}'] = p['retweet_counts']
                ws[f'J{max_row + 1}'] = p['source']
                media_ = ""
                if not isinstance(p['media'], str) and p['media'] is not None:
                    for media in p['media']:
                        media_ = f"{media_} {media.expanded_url};"
                    ws[f'K{max_row + 1}'] = media_
                else:
                    ws[f'K{max_row + 1}'] = p['media']
                user_mentions = ""
                if not isinstance(p['user_mentions'], str) and p['user_mentions'] is not None:
                    for user in p['user_mentions']:
                        user_mentions = f"{user_mentions} {user.screen_name};"
                    ws[f'L{max_row + 1}'] = user_mentions
                else:
                    ws[f'L{max_row + 1}'] = p['user_mentions']
                urls = ""
                if not isinstance(p['urls'], str):
                    for url in p['urls']:
                        urls = f"{urls} {url['expanded_url']};"
                    ws[f'M{max_row + 1}'] = urls
                else:
                    ws[f'M{max_row + 1}'] = p['urls']
                hashtags = ""
                if not isinstance(p['hashtags'], str):
                    for tag in p['hashtags']:
                        hashtags = f"{hashtags} {tag['text']};"
                    ws[f'N{max_row + 1}'] = hashtags
                else:
                    ws[f'N{max_row + 1}'] = p['hashtags']
                ws[f'O{max_row + 1}'] = p['symbols']
                max_row = max_row + 1
            if not filename:
                filename = f"searches-{self.keyword}.xlsx"
            wb.save(filename)
        else:
            return self.dict_['error']

    def to_csv(self, filename=None):
        all_tweets = self.dict_['tweets']
        if not filename:
            filename = f"searches-{self.keyword}.csv"
        with open(filename, "w", encoding="ASCII", errors="ignore", newline="") as csv_:
            writer = csv.writer(csv_)
            writer.writerow(WORKBOOK_HEADERS)
            for p in all_tweets:
                p = p.to_dict()
                created_on = p.get('created_at') if p.get('created_at') else p.get('created_on')
                author = p['author'].name
                is_retweet = p['is_retweet']
                is_reply = p['is_reply']
                tweet_id = p['tweet_id']
                tweet_body = p['tweet_body']
                language = p['language']
                likes = p['likes']
                retweet_count = p['retweet_counts']
                source = p['source']
                symbols = p['symbols']
                media_ = ""
                if not isinstance(p['media'], str) and p['media'] is not None:
                    for media in p['media']:
                        media_ = f"{media_} {media.expanded_url};"
                else:
                    media_ = p['media']
                user_mentions = ""
                if not isinstance(p['user_mentions'], str) and p['user_mentions'] is not None:
                    for user in p['user_mentions']:
                        user_mentions = f"{user_mentions} {user.screen_name};"
                else:
                    user_mentions = p['user_mentions']
                hashtags = ""
                if not isinstance(p['hashtags'], str):
                    for tag in p['hashtags']:
                        hashtags = f"{hashtags} {tag['text']};"
                else:
                    hashtags = p['hashtags']
                urls = ""
                if not isinstance(p['urls'], str):
                    for url in p['urls']:
                        urls = f"{urls} {url['expanded_url']};"
                else:
                    urls = p['urls']
                row = [created_on,author, is_retweet, is_reply, tweet_id, tweet_body, language, likes, retweet_count,
                       source, media_, user_mentions, urls, hashtags, symbols]
                writer.writerow(row)

    def to_dict(self):
        return self.dict_

    def __repr__(self):
        return f"<Search (keyword={self.keyword}) (count={len(self.dict_['tweets'])})>"


class Trends:
    def __init__(self,trends_dict):
        self.__dictionary = trends_dict
        self.name = self.__dictionary.get("name")
        self.url = self.__dictionary.get("url")
        self.tweet_count = self.__dictionary.get("tweet_count")

    def __repr__(self):
        return f"<Trends (name={self.name})>"

    def to_dict(self):
        return self.__dictionary


class UserLegacy:
    def __init__(self,user_dict):
        self.__dictionary = user_dict
        self.id = self.__dictionary.get("id")
        self.rest_id = self.__dictionary.get("id")
        self.created_at = self.__dictionary.get("created_at")
        self.default_profile = self.__dictionary.get("default_profile")
        self.default_profile_image = self.__dictionary.get("default_profile_image")
        self.description = self.__dictionary.get("description")
        self.entities = self.__dictionary.get("entities")
        self.fast_followers_count = self.__dictionary.get("fast_followers_count")
        self.favourites_count = self.__dictionary.get("favourites_count")
        self.followers_count = self.__dictionary.get("followers_count")
        self.friends_count = self.__dictionary.get("friends_count")
        self.has_custom_timelines = self.__dictionary.get("has_custom_timelines")
        self.is_translator = self.__dictionary.get("is_translator")
        self.listed_count = self.__dictionary.get("listed_count")
        self.location = self.__dictionary.get("location")
        self.media_count = self.__dictionary.get("media_count")
        self.name = self.__dictionary.get("name")
        self.normal_followers_count = self.__dictionary.get("normal_followers_count")
        self.profile_banner_url = self.__dictionary.get("profile_banner_url")
        self.profile_image_url_https = self.__dictionary.get("profile_image_url_https")
        self.profile_interstitial_type = self.__dictionary.get("profile_interstitial_type")
        self.protected = self.__dictionary.get("protected")
        self.screen_name = self.__dictionary.get("screen_name")
        self.statuses_count = self.__dictionary.get("statuses_count")
        self.translator_type = self.__dictionary.get("translator_type")
        self.verified = self.__dictionary.get("verified")

    def __repr__(self):
        return f"<User (id={self.rest_id}) (name={self.name}) (followers={self.followers_count}) (verified={self.verified})>"

    def to_dict(self):
        return self.__dictionary



class UserNotFound(Exception):
    """Exception raised when user isn't found.

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message="Either User not Found or is Restricted"):
        self.message = message
        super().__init__(self.message)


from distutils.core import setup

setup(
    name='tweety-ns',
    packages=['tweety'],
    version='0.2',
    license='MIT',
    description='An easy Twitter Scraper',
    author='Tayyab Kharl',
    author_email='tayyabmahr@gmail.com',
    url='https://github.com/mahrtayyab/tweety',
    keywords=['TWITTER', 'TWITTER SCRAPE', 'SCRAPE TWEETS'],
    install_requires=[
        'beautifulsoup4',
        'openpyxl',
        'requests'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3'
    ],
)
